#include "..\include\NRJacobi.h"

NRJacobi::NRJacobi(void)
{
}

NRJacobi::~NRJacobi(void)
{
}

bool NRJacobi::Jacobi(double **a, int n, double d[], double **v, int *nrot)
{
  int j,iq,ip,i;
  double tresh,theta,tau,t,sm,s,h,g,c,*b,*z;
  
  b=dvector(1,n);
  z=dvector(1,n);
  for (ip=1;ip<=n;ip++) {
    for (iq=1;iq<=n;iq++) v[ip][iq]=0.0;
    v[ip][ip]=1.0;
  }
  for (ip=1;ip<=n;ip++) {
    b[ip]=d[ip]=a[ip][ip];
    z[ip]=0.0;
  }
  *nrot=0;
  for (i=1;i<=50;i++) {
    sm=0.0;
    for (ip=1;ip<=n-1;ip++) {
      for (iq=ip+1;iq<=n;iq++)
        sm += fabs(a[ip][iq]);
    }
    if (sm == 0.0) {
      free_dvector(z,1,n);
      free_dvector(b,1,n);
      return true;
    }
    if (i < 4)
      tresh=0.2*sm/(n*n);
    else
      tresh=0.0;
    for (ip=1;ip<=n-1;ip++) {
      for (iq=ip+1;iq<=n;iq++) {
        g=100.0*fabs(a[ip][iq]);
        if (i > 4 && (double)(fabs(d[ip])+g) == (double)fabs(d[ip])
            && (double)(fabs(d[iq])+g) == (double)fabs(d[iq]))
          a[ip][iq]=0.0;
        else if (fabs(a[ip][iq]) > tresh) {
          h=d[iq]-d[ip];
          if ((double)(fabs(h)+g) == (double)fabs(h))
            t=(a[ip][iq])/h;
          else {
            theta=0.5*h/(a[ip][iq]);
            t=1.0/(fabs(theta)+sqrt(1.0+theta*theta));
            if (theta < 0.0) t = -t;
          }
          c=1.0/sqrt(1+t*t);
          s=t*c;
          tau=s/(1.0+c);
          h=t*a[ip][iq];
          z[ip] -= h;
          z[iq] += h;
          d[ip] -= h;
          d[iq] += h;
          a[ip][iq]=0.0;
          for (j=1;j<=ip-1;j++) {
            rot(a,s,tau,j,ip,j,iq);
            }
          for (j=ip+1;j<=iq-1;j++) {
            rot(a,s,tau,ip,j,j,iq);
            }
          for (j=iq+1;j<=n;j++) {
            rot(a,s,tau,ip,j,iq,j);
            }
          for (j=1;j<=n;j++) {
            rot(v,s,tau,j,ip,j,iq);
            }
          ++(*nrot);
        }
      }
    }
    for (ip=1;ip<=n;ip++) {
      b[ip] += z[ip];
      d[ip]=b[ip];
      z[ip]=0.0;
    }
  }
  free_dvector(z,1,n);
  free_dvector(b,1,n);
  nrerror("Too many iterations in routine jacobi");
  return false;
}

bool NRJacobi::Jacobi(float **a, int n, float d[], float **v, int *nrot)
{
  int j,iq,ip,i;
  float tresh,theta,tau,t,sm,s,h,g,c,*b,*z;
  
  b=vector(1,n);
  z=vector(1,n);
  for (ip=1;ip<=n;ip++) {
    for (iq=1;iq<=n;iq++) v[ip][iq]=0.0;
    v[ip][ip]=1.0;
  }
  for (ip=1;ip<=n;ip++) {
    b[ip]=d[ip]=a[ip][ip];
    z[ip]=0.0;
  }
  *nrot=0;
  for (i=1;i<=50;i++) {
    sm=0.0;
    for (ip=1;ip<=n-1;ip++) {
      for (iq=ip+1;iq<=n;iq++)
        sm += fabs(a[ip][iq]);
    }
    if (sm == 0.0) {
      free_vector(z,1,n);
      free_vector(b,1,n);
      return true;
    }
    if (i < 4)
      tresh=0.2f*sm/(n*n);
    else
      tresh=0.0;
    for (ip=1;ip<=n-1;ip++) {
      for (iq=ip+1;iq<=n;iq++) {
        g=100.0f*fabs(a[ip][iq]);
        if (i > 4 && (fabs(d[ip])+g) == fabs(d[ip])
            && (fabs(d[iq])+g) == fabs(d[iq]))
          a[ip][iq]=0.0;
        else if (fabs(a[ip][iq]) > tresh) {
          h=d[iq]-d[ip];
          if ((fabs(h)+g) == fabs(h))
            t=(a[ip][iq])/h;
          else {
            theta=0.5f*h/(a[ip][iq]);
            t=1.0f/(fabs(theta)+sqrt(1.0f+theta*theta));
            if (theta < 0.0) t = -t;
          }
          c=1.0f/sqrt(1.0f+t*t);
          s=t*c;
          tau=s/(1.0f+c);
          h=t*a[ip][iq];
          z[ip] -= h;
          z[iq] += h;
          d[ip] -= h;
          d[iq] += h;
          a[ip][iq]=0.0;
          for (j=1;j<=ip-1;j++) {
            rot(a,s,tau,j,ip,j,iq);
            }
          for (j=ip+1;j<=iq-1;j++) {
            rot(a,s,tau,ip,j,j,iq);
            }
          for (j=iq+1;j<=n;j++) {
            rot(a,s,tau,ip,j,iq,j);
            }
          for (j=1;j<=n;j++) {
            rot(v,s,tau,j,ip,j,iq);
            }
          ++(*nrot);
        }
      }
    }
    for (ip=1;ip<=n;ip++) {
      b[ip] += z[ip];
      d[ip]=b[ip];
      z[ip]=0.0;
    }
  }
  free_vector(z,1,n);
  free_vector(b,1,n);
  nrerror("Too many iterations in routine jacobi");
  return false;
}
