#include <afxwin.h>
#include "POT_MeshGenerator.h"
#include <stdlib.h>
#include <math.h>
#include <iostream>

////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
////////////////////////////////////////////////////////////////

using namespace std;
#define BALL_VALID_FILE "ball_valid_tmp_binary_file"

POT_MeshGenerator::POT_MeshGenerator(void)
{
}

POT_MeshGenerator::~POT_MeshGenerator(void)
{
}
void POT_MeshGenerator::createTriangles(POT_PointSet* bs)
{
	cout << ".";

	int i,j, k;

	//Conectivity using spheres

	POT_KdTree* tree = new POT_KdTree(bs);

	int baN = bs->m_pointN;

	int** node = new int*[baN];
	int* nodeN = new int[baN];

	for(i=0; i<baN; i++)
		nodeN[i] = 0;

	float (*point)[3] = bs->m_point;
	float *radius = bs->m_weight;

	for(i=0; i<baN; i++)
	{
		float* p = point[i];
		float r = radius[i];

		int listN, *list;
		tree->collectPointIndexInSphere(list, listN, p, 2.0f*r);

		for(j=0; j<listN; j++)
		{
			int i1 = list[j];
			if(i == i1)
				continue;
			float* p1 = point[i1];
			float r1 = radius[i1];

			float vx = p[0]-p1[0];
			float vy = p[1]-p1[1];
			float vz = p[2]-p1[2];
			double d = sqrt(vx*vx + vy*vy + vz*vz);

			if(r + r1 > d && fabs(r - r1) < d)
			{
				int* tmp = node[i];
				int nN = nodeN[i];
				int* n = node[i] = new int[nN+1];
				for(k=0; k<nN; k++)
					n[k] = tmp[k];
				if(nN != 0)
					delete[] tmp;
				n[k] = i1;
				nodeN[i]++;

				if(2*r1 < d)
				{
					tmp = node[i1];
					nN = nodeN[i1];
					n = node[i1] = new int[nN+1];
					for(k=0; k<nN; k++)
						n[k] = tmp[k];
					if(nN != 0)
						delete[] tmp;
					n[k] = i;
					nodeN[i1]++;
				}
			}
		}
	}
	delete tree;

	int triN_max = 3*baN;
	m_triN = 0;
	m_tris = new int[triN_max][3];

	//create candidates of triangles
	cout << "." ;
	for(i=0; i<baN; i++)
	{
		float* c1 = point[i];
		float r1 = radius[i];

		int* ni = node[i];
		int niN = nodeN[i];
		for(int n1=0; n1 < niN; n1++)
		{
			int j = ni[n1];
			if(j < i)
				continue;
			float* c2 = point[j];
			float r2 = radius[j];


			for(int n2=0; n2 < niN; n2++)
			{
				int k = ni[n2];
				if(k <= j)
					continue;

				float* c3 = point[k];
				float r3 = radius[k];

				float p1[3], p2[3];
				if(!intersectionOf3Balls(p1, p2, c1, c2, c3, r1, r2, r3))
					continue;

				//check wheather the intersection is most outside of balls
				bool flag1 = true;
				bool flag2 = true;
				for(int n3=0; n3 < niN; n3++)
				{
					int index = ni[n3];
					if(index == j || index == k)
					{
						continue;
					}
					float* c4 = point[index];
					float r4 = radius[index];
					r4 *= r4;

					if((p1[0]-c4[0])*(p1[0]-c4[0])+(p1[1]-c4[1])*(p1[1]-c4[1])+(p1[2]-c4[2])*(p1[2]-c4[2]) < r4)
						flag1 = false;
					if((p2[0]-c4[0])*(p2[0]-c4[0])+(p2[1]-c4[1])*(p2[1]-c4[1])+(p2[2]-c4[2])*(p2[2]-c4[2]) < r4)
						flag2 = false;

					if(!flag1 && !flag2)
						break;
				}

				if(!flag1 && !flag2)
					continue;

				if(m_triN == triN_max)
				{
					triN_max *= 2;
					int (*tris1)[3] = m_tris;
					m_tris = new int[triN_max][3];
					for(int o=0; o<m_triN; o++)
					{
						m_tris[o][0] = tris1[o][0];
						m_tris[o][1] = tris1[o][1];
						m_tris[o][2] = tris1[o][2];
					}
					delete[] tris1;
					//mesh->face = tris;
				}

				m_tris[m_triN][0] = i;
				m_tris[m_triN][1] = j;
				m_tris[m_triN][2] = k;
				m_triN++;
			}
		}
	}
	for(i=0; i<baN; i++)
		if(nodeN[i] != 0)
			delete node[i];
	delete[] node;
	delete[] nodeN;
}

void POT_MeshGenerator::createMesh(POT_PointSet* vs)
{
	int N = vs->m_pointN;
	m_mesh = new POT_Mesh(N, m_triN);

	float (*point)[3] = vs->m_point;
	int i;
	for(i=0; i<N; i++)
	{
		float* p = point[i];
		m_mesh->setVertex(i, p[0], p[1], p[2]);
	}
	delete vs;
	vs = NULL;

	for(i=0; i<m_triN; i++)
	{
		int* t = m_tris[i];
		m_mesh->setFace(i, t[0], t[1], t[2]);
	}
	delete[] m_tris;
	m_tris = m_mesh->m_face;
}

void POT_MeshGenerator::cleanBadTriangles()
{
	cout << ".";

	int i,j;

	int N = m_mesh->m_vertexN;

	bool* decideT = new bool[m_triN];
	bool* decideV = new bool[N];
	int** link = new int*[N];
	int* linkN = new int[N];

	for(i=0; i<N; i++)
	{
		link[i] = NULL;
		linkN[i] = 0;
		decideV[i] = false;
	}

	//create the list of triangles incident to each vertex
	for(i=0; i<m_triN; i++)
	{
		decideT[i] = false;
		for(j=0; j<3; j++)
		{
			int index = m_tris[i][j];

			int lN = linkN[index];
			int* l_tmp = new int[lN+1];
			int* l = link[index];
			for(int k=0; k<lN; k++)
				l_tmp[k] = l[k];
			l_tmp[lN] = i;
			if(lN != 0)
				delete[] l;
			link[index] = l_tmp;
			linkN[index]++;
		}
	}

	int* visitID = new int[N];
	for(i=0; i<N; i++)
		visitID[i] = -1;

	//Find seed 1-ring
	cout << ".";
	float min_error = 10000000000.0f;
	int seed = -1;
	for(i=0; i<N; i++)
	{
		if(linkN[i] == 0)
			continue;
		if(sortOneRings(decideT, link[i], linkN[i], i, visitID))
		{
			float e = measureOneRingError(link[i], linkN[i], i);
			if(e < min_error && e > 0)
			{
				seed = i;
				min_error = e;
			}
		}
	}

	if(seed < 0)
	{
		cout << "Seed 1 -ring can not be found." << endl;
		cout << "Maybe, the input points contains too large noise." << endl;
		cout << "Now exiting, sorry." << endl;
		return;
	}

	delete[] visitID;
	visitID = new int[m_triN];
	for(i=0; i<m_triN; i++)
		visitID[i] = -1;

	bool* validT = new bool[m_triN];
	for(i=0; i<m_triN; i++)
		validT[i] = true;

	for(i=0; i<m_triN; i++)
		decideT[i] = false;
	sortOneRings(decideT, link[seed], linkN[seed], seed, visitID);

	//compute global orientation using priority Q
	cout << "." ;
	float* cost = new float[N];

	int* heap = new int[N+1];
	int* index = new int[N];
	int last_heap = 0;

	for(i=0; i<N; i++)
	{
		cost[i] = 1000000000000.0f;
		index[i] = -1;
	}

	int *best, bestN;
	getBestOneRing(best, bestN, cost[seed],	decideT, validT, link[seed], linkN[seed], seed, visitID);
	delete best;
	//insert
	heap[++last_heap] = seed;
	index[seed] = last_heap;
	upheap(cost, last_heap, last_heap, heap, index);
	int counter = 0;
	while(last_heap > 0)
	{
		//remove top
		int current = heap[1];
		index[current] = -1;
		heap[1] = heap[last_heap--];
		index[heap[1]] = 1;
		downheap(cost, last_heap, 1, heap, index);


		int* lc = link[current];
		int lcN = linkN[current];
		if(!getBestOneRing(best, bestN, cost[current], decideT, validT, lc, lcN, current, visitID))
		{
			continue;
		}

		//decide vertex and triangles
		for(j=0; j<lcN; j++)
			decideT[lc[j]] = false;

		//decide vertex and triangles
		decideV[current] = true;
		for(j=0; j<bestN; j++)
			decideT[best[j]] = true;

		//remove un-used triangles
		for(j=0; j<lcN; j++)
			if(!decideT[lc[j]])
				validT[lc[j]] = false;

		//replace trinagle list
		delete link[current];
		link[current] = best;
		linkN[current] = bestN;

		//update or insert vertices
		for(j=0; j<bestN; j++)
		{
			int* ti = m_tris[best[j]];
			int v;
			if(ti[0] == current)
				v = ti[1];
			else if(ti[1] == current)
				v = ti[2];
			else
				v = ti[0];

			if(decideV[v])
				continue;

			if(index[v] < 0)
			{
				//insert new vertex
				int* bv;
				int bvN;
				if(getBestOneRing(bv, bvN, cost[v],decideT, validT, link[v], linkN[v], v, visitID))
				{
					heap[++last_heap] = v;
					index[v] = last_heap;
					upheap(cost, last_heap, last_heap, heap, index);
					delete bv;
				}
			}
		}
	}

	delete[] heap;
	delete[] index;
	delete[] cost;

	delete[] visitID;

	delete[] validT;

	for(i=0; i<N; i++)
		if(linkN[i] != 0)
			delete[] link[i];
	delete[] link;
	delete[] linkN;

	cout << ".";
	while(!removeNonManifoldV(decideT));

	delete[] decideV;

//	while(true)
	{

		int valid_triN = 0;
		for(i=0; i<m_triN; i++)
		{
			if(!decideT[i])
				continue;

			int* f = m_tris[i];

			m_tris[valid_triN][0] = f[0];
			m_tris[valid_triN][1] = f[1];
			m_tris[valid_triN][2] = f[2];

			valid_triN++;
		}
		m_triN = m_mesh->m_faceN = valid_triN;
		delete[] decideT;

		cout << "." ;

		int* table = new int[N];
		for(i=0; i<N; i++)
		table[i] = -1;
		int id = 0;
		for(i=0; i<m_triN; i++)
		{
			int* f = m_tris[i];
			for(j=0; j<3; j++)
				if(table[f[j]] < 0)
					table[f[j]] = id++;
		}
		float (*vertex_tmp)[3] = new float[id][3];
		FILE* ball_valid_out = fopen(BALL_VALID_FILE, "w+b");
		fwrite(table, sizeof(int), N, ball_valid_out);
		fclose(ball_valid_out);

		for(i=0; i<N; i++)
		{
			int j = table[i];
			if(j < 0)
				continue;
			vertex_tmp[j][0] = m_mesh->m_vertex[i][0];
			vertex_tmp[j][1] = m_mesh->m_vertex[i][1];
			vertex_tmp[j][2] = m_mesh->m_vertex[i][2];
		}

		delete[] m_mesh->m_vertex;
		m_mesh->m_vertex = vertex_tmp;
		m_mesh->m_vertexN = N = id;

		for(i=0; i<m_triN; i++)
		{
			int* f = m_tris[i];
			f[0] = table[f[0]];
			f[1] = table[f[1]];
			f[2] = table[f[2]];
		}
		delete[] table;

		m_mesh->fillSmallHolesMinA(10);

		m_tris = m_mesh->m_face;
		m_triN = m_mesh->m_faceN;
	}
}
bool POT_MeshGenerator::intersectionOf3Balls(float p1[3], float p2[3],
                        float c1[3], float c2[3], float c3[3],
                        float r1, float r2, float r3)
{
	float vx1 = c2[0] - c1[0];
	float vy1 = c2[1] - c1[1];
	float vz1 = c2[2] - c1[2];
	float d2 = vx1*vx1 + vy1*vy1 + vz1*vz1;
	float d = (float)sqrt(d2);
	vx1 /= d;
	vy1 /= d;
	vz1 /= d;
	if(r1 + r2 - d <= 0 || d <= fabs(r1-r2))
		return false;

	float w1 = d2 + r2*r2 - r1*r1;
	float w2 = d2 + r1*r1 - r2*r2;
	float w = 2*d2;
	w1 /= w;
	w2 /= w;

	float cx1 = w1*c1[0] + w2*c2[0];
	float cy1 = w1*c1[1] + w2*c2[1];
	float cz1 = w1*c1[2] + w2*c2[2];

	float d1 = d*w2;
	float R1 = (float)sqrt(r1*r1 - d1*d1);

	float dot = vx1*(c3[0]-cx1) + vy1*(c3[1]-cy1) + vz1*(c3[2]-cz1);
	if(fabs(dot) >= r3)
		return false;

	float cx2 = c3[0] - dot*vx1;
	float cy2 = c3[1] - dot*vy1;
	float cz2 = c3[2] - dot*vz1;
	float R2 = (float)sqrt(r3*r3 - dot*dot);

	float vx = cx2 - cx1;
	float vy = cy2 - cy1;
	float vz = cz2 - cz1;
	d2 = vx*vx + vy*vy + vz*vz;
	d = (float)sqrt(d2);

	if(R1 + R2 - d <= 0 || d <= fabs(R1-R2))
		return false;

	w1 = d2 + R2*R2 - R1*R1;
	w2 = d2 + R1*R1 - R2*R2;
	w = 2*d2;
	w1 /= w;
	w2 /= w;

	float cx = w1*cx1 + w2*cx2;
	float cy = w1*cy1 + w2*cy2;
	float cz = w1*cz1 + w2*cz2;

	float nx = vy*vz1 - vz*vy1;
	float ny = vz*vx1 - vx*vz1;
	float nz = vx*vy1 - vy*vx1;
	float len = (float)sqrt(nx*nx + ny*ny + nz*nz);
	if(len == 0)
		return false;
	nx /= len;
	ny /= len;
	nz /= len;

	d1 = d*w2;
	float h = (float)sqrt(R1*R1 - d1*d1);

	p1[0] = cx + h*nx;
	p1[1] = cy + h*ny;
	p1[2] = cz + h*nz;

	p2[0] = cx - h*nx;
	p2[1] = cy - h*ny;
	p2[2] = cz - h*nz;

	return true;
}
bool POT_MeshGenerator::sortOneRings(bool* decideT, int* list, int N, int current, int* visitID)
{
	if(N < 3)
		return false;

	int i;
	int (*tris)[3] = m_mesh->m_face;

	int start = list[0];
	for(i=1; i<N; i++)
		if(decideT[list[i]])
		{
			start = list[i];
			break;
		}

	int end = start;
	do{
		int* ti = tris[end];
		int v;
		if(ti[0] == current)
			v = ti[1];
		else if(ti[1] == current)
			v = ti[2];
		else
			v = ti[0];

		if(visitID[v] == current)
			return false;
		visitID[v] = current;

		bool flag = false;
		int next;
		for(i=0; i<N; i++)
		{
			int s = list[i];
			if(s == end)
				continue;
			ti = tris[s];
			int ver_id;
			if(ti[0] == v)
				ver_id = 0;
			else if(ti[1] == v)
				ver_id = 1;
			else if(ti[2] == v)
				ver_id = 2;
			else
				continue;
			if(flag)
				return false;
			flag = true;
			next = s;
			int i1 = (ver_id+1)%3;
			if(ti[i1] != current)
			{
				int i2 = (ver_id+2)%3;
				int tmp = ti[i1];
				ti[i1] = ti[i2];
				ti[i2] = tmp;
			}
		}
		if(!flag)
			return false;
		end = next;
	}while(start != end);

	for(i=0; i<N; i++)
		decideT[list[i]] = true;

	return true;
}
bool POT_MeshGenerator::getBestOneRing(int* &blist, int &blistN, float& error,
                bool* decideT, bool* validT,
                int* list, int N, int current, int* visitID)
{
	int (*tris)[3] = m_mesh->m_face;

	blist = NULL;
	blistN = 0;

	int i;
	int* stack = new int[N+1];
	int top = -1;

	int start = -1;
	for(i=0; i<N; i++)
	{
		if(decideT[list[i]] && validT[list[i]])
		{
			start = i;
			break;
		}
	}
	if(start < 0)
	{
		delete[] stack;
		return false;
	}

	int startV;
	int* ti = tris[list[start]];
	if(ti[0] == current)
		startV = ti[2];
	else if(ti[1] == current)
		startV = ti[0];
	else
		startV = ti[1];

	stack[++top] = start;
	stack[++top] = -1;
	while(top > 0)
	{
		int v;
		int t_id = list[stack[top-1]];
		int* ti = tris[t_id];
		if(ti[0] == current)
			v = ti[1];
		else if(ti[1] == current)
			v = ti[2];
		else
			v = ti[0];

		if(v == startV)
		{
			int* list_tmp = new int[top];
			for(i=0; i<top; i++)
				list_tmp[i] = list[stack[i]];

			bool flag = true;
			for(i=0; i<N; i++)
			{
				if(!decideT[list[i]])
					continue;
				bool flag1 = false;
				for(int j=0; j<top; j++)
				{
					if(list_tmp[j] == list[i])
					{
						flag1 = true;
						break;
					}
				}
				if(!flag1)
				{
					flag = false;
					break;
				}
			}

			if(flag)
			{
				float error_tmp = measureOneRingError(list_tmp, top, current);
				if(blistN == 0 || error_tmp < error)
				{
					error = error_tmp;
					delete blist;
					blist = list_tmp;
					blistN = top;
				}
				else
					delete list_tmp;
			}
			else
				delete list_tmp;
			top--;
			continue;
		}

		visitID[t_id] = current;

		bool flag = false;
		if(stack[top] < 0)
			i = 0;
		else
			i = stack[top] + 1;

		top--;
		for(; i<N; i++)
		{
			if(i == stack[top] || visitID[list[i]] == current || !validT[list[i]])
				continue;

			ti = tris[list[i]];
			int ver_id;
			if(ti[0] == v)
				ver_id = 0;
			else if(ti[1] == v)
				ver_id = 1;
			else if(ti[2] == v)
				ver_id = 2;
			else
				continue;

			if(decideT[list[i]])
			{
				int i1 = (ver_id+1)%3;
				if(ti[i1] != current)
					continue;
			}

			flag = true;
			stack[++top] = i;
			int i1 = (ver_id+1)%3;
			if(ti[i1] != current)
			{
				int i2 = (ver_id+2)%3;
				int tmp = ti[i1];
				ti[i1] = ti[i2];
				ti[i2] = tmp;
			}
			break;
		}
		if(flag)
			stack[++top] = -1;
		else
			visitID[t_id] = -1;
	}
	delete[] stack;

	if(blistN < 3)
		return false;

	for(i=0; i<blistN; i++)
	{
		int j = blist[i];

		if(i+1 == blistN)
			continue;

		ti = tris[j];
		int v;
		if(ti[0] == current)
			v = ti[1];
		else if(ti[1] == current)
			v = ti[2];
		else
			v = ti[0];

		ti = tris[blist[i+1]];
		int ver_id;
		if(ti[0] == v)
			ver_id = 0;
		else if(ti[1] == v)
			ver_id = 1;
		else
			ver_id = 2;

		int i1 = (ver_id+1)%3;
		if(ti[i1] != current)
		{
			int i2 = (ver_id+2)%3;
			int tmp = ti[i1];
			ti[i1] = ti[i2];
			ti[i2] = tmp;
		}
	}
	return true;
}
  
float POT_MeshGenerator::measureOneRingError(int* list, int N, int current)
{
	float (*verts)[3] = m_mesh->m_vertex;
	int (*tris)[3] = m_mesh->m_face;

	if(N < 3)
		return 100000.0f;
	double (*normal)[3] = new double[N][3];
	double *area = new double[N];
	int i = 0;
	for(i=0; i<N; i++)
	{
		int* ti = tris[list[i]];
		float *p0, *p1, *p2;
		if(ti[0] == current)
		{
			p0 = verts[ti[0]];
			p1 = verts[ti[1]];
			p2 = verts[ti[2]];
		}
		else if(ti[1] == current)
		{
			p0 = verts[ti[1]];
			p1 = verts[ti[2]];
			p2 = verts[ti[0]];
		}
		else
		{
			p0 = verts[ti[2]];
			p1 = verts[ti[0]];
			p2 = verts[ti[1]];
		}

		float v1[] = {p1[0]-p0[0], p1[1]-p0[1], p1[2]-p0[2]};
		float v2[] = {p2[0]-p0[0], p2[1]-p0[1], p2[2]-p0[2]};

		double *n = normal[i];
		n[0] = v1[1]*v2[2] - v1[2]*v2[1];
		n[1] = v1[2]*v2[0] - v1[0]*v2[2];
		n[2] = v1[0]*v2[1] - v1[1]*v2[0];
		double len = sqrt(n[0]*n[0] + n[1]*n[1] + n[2]*n[2]);
		if((float)len != 0)
		{
			n[0] /= len;
			n[1] /= len;
			n[2] /= len;
		}
		area[i] = sqrt(v1[0]*v1[0] + v1[1]*v1[1] + v1[2]*v1[2]);
	}
	double error = 0;
	for(i=0; i<N; i++)
	{
		double* n1 = normal[i];
		double* n2 = normal[(i+1)%N];
		double dot = n1[0]*n2[0] + n1[1]*n2[1] + n1[2]*n2[2];
		if(dot < -1.0)
			dot = -1.0;
		else if(dot > 1.0)
			dot = 1.0;

		float a = (float)acos(dot);
		if(a > error)
			error = a;
		continue;

		error += (area[i]/* + area[(i+1)%N]*/)*acos(dot);
	}
	delete[] normal;
	delete[] area;
	return (float)error;
}
bool POT_MeshGenerator::removeNonManifoldV(bool* decideT)
{
	int i, j, k;

	bool isOK = true;

	int N = m_mesh->m_vertexN;
	int *ver_to_triN = new int[N];
	int** ver_to_tri = new int*[N];

	while(true)
	{
		for(i=0; i<N; i++)
			ver_to_triN[i] = 0;

		for(i=0; i<m_triN; i++)
		{
			if(!decideT[i])
				continue;
			int* t = m_tris[i];
			ver_to_triN[t[0]]++;
			ver_to_triN[t[1]]++;
			ver_to_triN[t[2]]++;
		}

		bool isChanged = false;
		for(i=0; i<m_triN; i++)
		{
			if(!decideT[i])
				continue;
			int* t = m_tris[i];
			for(j=0; j<3; j++)
			{
				if(ver_to_triN[t[j]] == 1)
				{
					decideT[i] = false;
					isChanged = true;
				}
			}
		}

		if(!isChanged)
			break;
		else
			isOK = false;
	}

	for(i=0; i<N; i++)
	{
		if(ver_to_triN[i] != 0)
		{
			ver_to_tri[i] = new int[ver_to_triN[i]];
			ver_to_triN[i] = 0;
		}
	}

	for(i=0; i<m_triN; i++)
	{
		if(!decideT[i])
			continue;
		int* t = m_tris[i];
		for(j=0; j<3; j++)
		{
			k = t[j];
			ver_to_tri[k][ver_to_triN[k]++] = i;
		}
	}

	for(i=0; i<N; i++)
	{
		int vtN = ver_to_triN[i];
		if(vtN < 2)
			continue;
		int* vt = ver_to_tri[i];
		int start = 0;
		for(j=0; j<vtN; j++)
		{
			int* t = m_tris[vt[j]];
			int v;
			if(t[0] == i)
				v = t[1];
			else if(t[1] == i)
				v = t[2];
			else
				v = t[0];
			for(k=0; k<vtN; k++)
			{
				if(k == j)
					continue;
				int* t1 = m_tris[vt[k]];
				if(t1[0] == v || t1[1] == v || t1[2] == v)
					break;
			}
			if(k == vtN)
			{
				start = j;
				break;
			}
		}

		int* t = m_tris[vt[start]];
		int startV;
		if(t[0] == i)
			startV = t[1];
		else if(t[1] == i)
			startV = t[2];
		else
			startV = t[0];
		int remove = -1;

		for(j=0; j<vtN-1; j++)
		{
			t = m_tris[vt[(j+start)%vtN]];
			int v;
			if(t[0] == i)
				v = t[2];
			else if(t[1] == i)
				v = t[0];
			else
				v = t[1];

			if(v == startV)
			{
				remove = j+1;
				break;
			}

			int index = -1;
			for(k=0; k<vtN; k++)
			{
				if(k == j)
					continue;
				int* t1 = m_tris[vt[(k+start)%vtN]];
				int v1;
				if(t1[0] == i)
					v1 = t1[1];
				else if(t1[1] == i)
					v1 = t1[2];
				else
					v1 = t1[0];
				if(v1 == v)
				{
					if(index < 0)
						index = k;
					else
					{
						remove = j;
						break;
					}
				}
			}
			if(index < 0)
				remove = j+1;

			if(remove >= 0)
				break;

			int tmp = vt[(j+1+start)%vtN];
			vt[(j+1+start)%vtN] = vt[(index+start)%vtN];
			vt[(index+start)%vtN] = tmp;
		}

		if(remove >= 0)
		{
			for(j=remove; j<vtN; j++)
				decideT[vt[(j+start)%vtN]] = false;
			isOK = false;
		}
	}

	for(i=0; i<N; i++)
		if(ver_to_triN[i] != 0)
			delete[] ver_to_tri[i];
	delete[] ver_to_tri;
	delete[] ver_to_triN;

	return isOK;
}
int* POT_MeshGenerator::getValidBallTable(int baN)
{
	int *table = new int[baN];
	FILE* ball_valid_out = fopen(BALL_VALID_FILE, "r+b");
	fread(table, sizeof(int), baN, ball_valid_out);
	fclose(ball_valid_out);
	return table;
}

int POT_MeshGenerator::removeTempFile()
{
  return remove(BALL_VALID_FILE);
}