#include <afxwin.h>
#include "POT_SphereGenerator.h"
#include "NRiCPP/include/NR.h"
#include <math.h>
////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
////////////////////////////////////////////////////////////////////
#define MAX_BISECTION 20
#define CONV_BISECTION 0.0001f
#define K 10
#define MCN 15
#define Tover 1000000.0f
#define Nmin 6
#define BALL_FILE "ball_tmp_binary_file"
#define VERTEX_FILE "vertex_tmp_binary_file"
#define NORMAL_FILE "normal_tmp_binary_file"

#define OUT_NUMBER 10000

using namespace std;

POT_SphereGenerator::POT_SphereGenerator(POT_PointSet* ps)
{
	this->m_ps = ps;
	m_tree = new POT_KdTree(ps);

	m_isSelected = NULL;	m_isSelectedNeigh = NULL;
	m_ballPts = NULL;
	m_vertPts = NULL;
	m_selectNum = m_neighNum = 0;
	m_duration = 0;
	m_recomputeNormal = true;
}

POT_SphereGenerator::~POT_SphereGenerator(void)
{
	delete m_tree;
	if(m_isSelected != NULL)	delete m_isSelected;
	if(m_isSelectedNeigh != NULL) delete m_isSelectedNeigh;
	if(m_ballPts != NULL)	delete m_ballPts;
	if(m_vertPts != NULL) delete m_vertPts;
}
POT_PointSet* POT_SphereGenerator::getBalls2()
{
	float *buff = new float[4*m_baN];
	m_ball_out = fopen(BALL_FILE, "r+b");
	fread(buff, sizeof(float), 4*m_baN, m_ball_out);
	fclose(m_ball_out);

	POT_PointSet* bs = new POT_PointSet(m_baN);
	int i;
	int j = 0;
	for(i=0; i<m_baN; i++)
	{
		float x = buff[j++];
		float y = buff[j++];
		float z = buff[j++];
		float r = buff[j++];

		bs->setPoint(i, x, y, z);
		bs->setWeight(i, r);
	}
	delete[] buff;

	buff = new float[3*m_baN];
	m_normal_out = fopen(NORMAL_FILE, "r+b");
	fread(buff, sizeof(float), 3*m_baN, m_normal_out);
	fclose(m_normal_out);

	j = 0;
	for(i=0; i<m_baN; i++)
	{
		float x = buff[j++];
		float y = buff[j++];
		float z = buff[j++];

		bs->setNormal(i, x, y, z);	  
	}
	delete[] buff;
	return bs;
}

POT_PointSet* POT_SphereGenerator::getBalls()
{
	float *buff = new float[4*m_baN];
	m_ball_out = fopen(BALL_FILE, "r+b");
	fread(buff, sizeof(float), 4*m_baN, m_ball_out);
	fclose(m_ball_out);

	POT_PointSet* bs = new POT_PointSet(m_baN);
	int i;
	int j = 0;
	for(i=0; i<m_baN; i++)
	{
		float x = buff[j++];
		float y = buff[j++];
		float z = buff[j++];
		float r = buff[j++];

		bs->setPoint(i, x, y, z);
		bs->setWeight(i, r);
	}
	delete[] buff;
	return bs;
}
  
POT_PointSet* POT_SphereGenerator::getVertices()
{
	float *buff = new float[3*m_baN];
	m_vertex_out = fopen(VERTEX_FILE, "r+b");
	fread(buff, sizeof(float), 3*m_baN, m_vertex_out);
	fclose(m_vertex_out);

	POT_PointSet* vs = new POT_PointSet(m_baN);
	int i;
	int j = 0;
	for(i=0; i<m_baN; i++)
	{
		float x = buff[j++];
		float y = buff[j++];
		float z = buff[j++];

		vs->setPoint(i, x, y, z);
	}
	delete[] buff;
	return vs;
}
  
void POT_SphereGenerator::precom()
{
	float max[3], min[3];
	m_ps->getBound(min, max);
	float sx = max[0] - min[0];
	float sy = max[1] - min[1];
	float sz = max[2] - min[2];
	m_dia2 = sx*sx + sy*sy + sz*sz;
	m_dia = sqrt(m_dia2);

	computeWeightAndNormal();
}
void POT_SphereGenerator::generate(float err, float ave, float qem)
{
	m_Terr = err*m_dia;
	m_Terr2 = m_totalA*m_Terr*m_Terr;
	m_Tave = (float)(m_dia*ave);
	m_Tqem = qem;
	m_TTNmin = (int)(qem*qem*Nmin);

	m_baN = 0;
	m_ball_out = fopen(BALL_FILE, "w+b");
	m_vertex_out = fopen(VERTEX_FILE, "w+b");
	m_normal_out = fopen(NORMAL_FILE, "w+b");
	generateBalls();
	fclose(m_ball_out);
	fclose(m_vertex_out);
	fclose(m_normal_out); 
}
void POT_SphereGenerator::computeWeightAndNormal()
{
	bool computeN =  m_recomputeNormal || (m_ps->m_normal == NULL);

	int i, j, k;

	int N = m_ps->m_pointN;
	float (*point)[3] = m_ps->m_point;
	float *weight = m_ps->m_weight;

	float size = m_tree->getLeafSize();
	double *d_array = new double[K];
	int *p_array = new int[K+1];
	m_totalA = 0;
	for(i=0; i<N; i++)
	{
		float* p = point[i];
		float r = size;
		int listN, *list;
		do{
			m_tree->collectPointIndexInSphere(list, listN, p, r);
			r *= 1.5f;
		}while(listN < K+1);

		for(j=0; j<K; j++)
			d_array[j] = m_dia2;

		for(j=0; j<listN; j++)
		{
			float* q = point[list[j]];
			if(p == q)
				continue;
			float vx = q[0] - p[0];
			float vy = q[1] - p[1];
			float vz = q[2] - p[2];
			double d = vx*vx + vy*vy + vz*vz;

			int index = -1;
			for(k=0; k<K; k++)
			{
				if(d < d_array[k])
				{
					index = k;
					break;
				}
			}
			if(index < 0)
				continue;

			for(k=K-1; k>index; k--)
			{
				p_array[k] = p_array[k-1];
				d_array[k] = d_array[k-1];
			}
			p_array[index] = list[j];
			d_array[index] = d;
		}
		double a = 0;
		for(j=0; j<K; j++)
			a += d_array[j];
		a = a/K;
		weight[i] *= (float)a;
		m_totalA += a;

		if(computeN)
		{
			p_array[K] = i;
			float n[3];
			computeNormalWithCV(n, p_array, K+1);
			m_ps->setNormal(i, n[0], n[1], n[2]);
		}
	}
	delete[] p_array;
	delete[] d_array;
}

void POT_SphereGenerator::generateBalls()
{

	clock_t start = clock();

	int i,j;
	int N = m_ps->m_pointN;
	float (*center)[3] = m_ps->m_point;
	float (*normal)[3] = m_ps->m_normal;

	POT_KdTree* c_tree = m_tree;
	POT_PointSet* cs = m_ps;

	int tableN = N;

	float* overlap = new float[tableN];
	int* active_table = new int[tableN];
	int activeN = 0;

	int t_baN = 0;
	POT_Sphere** t_ba = new POT_Sphere*[OUT_NUMBER];

	if(m_Tave != 0)
	{
		cout << "." ;
		float *weight = m_ps->m_weight;
		float (*normal)[3] = m_ps->m_normal;
		POT_PointSet* sps = new POT_PointSet(N);
		for(i=0; i<N; i++)
		{
			float* p = center[i];

			int listN, *list;
			c_tree->collectPointIndexInSphere(list, listN, p, m_Tave);
			float totalW = 0;
			float c[3] = {0,0,0};
			for(j=0; j<listN; j++)
			{
				int k = list[j];
				float* q = center[k];
				float w = weight[k];
				c[0] += w*q[0];
				c[1] += w*q[1];
				c[2] += w*q[2];
				totalW += w;
			}
			c[0] /= totalW;
			c[1] /= totalW;
			c[2] /= totalW;

			sps->setPoint(i, c[0], c[1], c[2]);
		}
		cs = sps;
		center = sps->m_point;
		c_tree = new POT_KdTree(sps);
	}

	for(i=0; i<N; i++)
	{
		active_table[activeN++] = i;
		overlap[i] = 0;
	}

	cout << "." ;
	N = tableN = activeN;

	int* can = new int[MCN];
	long idum = 1;

	//For counter to stdout
	int pre_activeN = activeN;
	int percent = 0;

	while(activeN > 0)
	{
		int index;
		if(activeN > MCN)
		{
			for(j=0; j<MCN; j++)
			{
				int k =(int)(ran0(&idum)*(tableN-1)); //(tableN-1)/2;//
				can[j] = active_table[k];
				if(overlap[can[j]] >= Tover)
					j--;
			}

			//search minimun from multiple-choice
			index = can[0];
			for(j=1; j<MCN; j++)
			{
				if(overlap[index] > overlap[can[j]])
					index = can[j];
			}
		}
		else
		{
			index = -1;
			for(j=0; j<tableN; j++)
			{
				int k = active_table[j];
				if(overlap[k] >= Tover)
					continue;
				if(index < 0 || overlap[index] > overlap[k])
					index = k;
			}
			if(index < 0)
				break;
		}

		float* c = center[index];
		overlap[index] = Tover;
		activeN--;

		//Compute a ball
		POT_Sphere* ba = new POT_Sphere(c[0], c[1], c[2]);

		ba->m_nx = normal[index][0];	
		ba->m_ny = normal[index][1];
		ba->m_nz = normal[index][2];

		while(true)
		{
			computeOptimalSupportL2(ba);
			break;
		}
		float r = ba->m_r;
		//updtae overlap
		int *list, listN;
		c_tree->collectPointIndexInSphere(list, listN, c, r);

		bool *onHull = new bool[listN];
		computeHull(onHull, list, listN, c);
		for(j=0; j<listN; j++)
		{
			int k = list[j];
			if(overlap[k] >= Tover)
				continue;
			float *p = center[k];
			if(!onHull[j])
			{
				overlap[k] = Tover;
				if(overlap[k] >= Tover)
				{
					activeN--;
				}
			}
			else
			overlap[k] += (float)(ba->weight(p[0], p[1], p[2]));
		}
		delete[] onHull;      

		//Write the ball to temp file
		m_baN++;
		t_ba[t_baN++] = ba;
		if(t_baN%OUT_NUMBER == 0)
		{
			float *buff = new float[4*OUT_NUMBER];
			int k = 0;
			for(j=0; j<OUT_NUMBER; j++)
			{
				POT_Sphere* b = t_ba[j];
				buff[k++] = b->m_cx;
				buff[k++] = b->m_cy;
				buff[k++] = b->m_cz;
				buff[k++] = b->m_r;
			}
			fwrite(buff, sizeof(float), 4*OUT_NUMBER, m_ball_out);
			delete[] buff;

			buff = new float[3*OUT_NUMBER];
			k = 0;
			for(j=0; j<OUT_NUMBER; j++)
			{
				POT_Sphere* b = t_ba[j];
				buff[k++] = b->m_nx;
				buff[k++] = b->m_ny;
				buff[k++] = b->m_nz;
			}
			fwrite(buff, sizeof(float), 3*OUT_NUMBER, m_normal_out);
			delete[] buff;

			buff = new float[3*OUT_NUMBER];
			k = 0;
			for(j=0; j<OUT_NUMBER; j++)
			{
				POT_Sphere* b = t_ba[j];
				buff[k++] = b->m_px;
				buff[k++] = b->m_py;
				buff[k++] = b->m_pz;
				delete b;
			}
			fwrite(buff, sizeof(float), 3*OUT_NUMBER, m_vertex_out);
			delete[] buff;

			t_baN = 0;
		}

		if(activeN < tableN/2)
		{ //resize table
			int count = 0;
			for(j=0; j<tableN; j++)
			{
				int k = active_table[j];
				if(overlap[k] < Tover)
				{
					active_table[count++] = k;
				}
			}
			tableN = count;
		}      
		pre_activeN = activeN;
	}
	delete[] can;
	delete[] active_table;
	delete[] overlap;

	if(m_Tave != 0)
	{
		delete c_tree;
		delete cs;
	}

	m_duration += clock()-start;

	if(t_baN != 0)
	{
		float *buff = new float[4*t_baN];
		int k = 0;
		for(j=0; j<t_baN; j++)
		{
			POT_Sphere* b = t_ba[j];

			buff[k++] = b->m_cx;
			buff[k++] = b->m_cy;
			buff[k++] = b->m_cz;
			buff[k++] = b->m_r;
		}
		fwrite(buff, sizeof(float), 4*t_baN, m_ball_out);
		delete[] buff;

		buff = new float[3*t_baN];
		k = 0;
		for(j=0; j<t_baN; j++)
		{
			POT_Sphere* b = t_ba[j];

			buff[k++] = b->m_nx;
			buff[k++] = b->m_ny;
			buff[k++] = b->m_nz;
		}
		fwrite(buff, sizeof(float), 3*t_baN, m_normal_out);
		delete[] buff;

		buff = new float[3*t_baN];
		k = 0;
		for(j=0; j<t_baN; j++)
		{
			POT_Sphere* b = t_ba[j];

			buff[k++] = b->m_px;
			buff[k++] = b->m_py;
			buff[k++] = b->m_pz;

			delete b;
		}
		fwrite(buff, sizeof(float), 3*t_baN, m_vertex_out);
		delete[] buff;
	}
	delete[] t_ba;
}
void POT_SphereGenerator::computeOptimalSupportL2(POT_Sphere* ba)
{
	int i;
	float scaleL = (float)(10*m_Terr);
	float scaleS = (float)(m_Terr);
	double errorL, errorS;

	ba->m_r = scaleS;
	errorS = localFitQ(ba);
	if(errorS > m_Terr2)
	{
		while(errorS > m_Terr2)
		{
			scaleL = scaleS;
			scaleS *= 0.5f;
			errorL = errorS;
			ba->m_r = scaleS;
			errorS = localFitQ(ba);
		}
	}
	else
	{
		ba->m_r = scaleL;
		errorL = localFitQ(ba);
		while(errorL < m_Terr2)
		{
			scaleS = scaleL;
			scaleL *= 2;
			errorS = errorL;
			ba->m_r = scaleL;
			errorL = localFitQ(ba);
		}
	}

	for(i=0; i<MAX_BISECTION; i++)
	{
		float r = 0.5f*(scaleS + scaleL);
		ba->m_r = r;
		double e = localFitQ(ba);

		if(e < m_Terr2)
		{
			scaleS = r;
			errorS = e;
		}
		else
		{
			scaleL = r;
			errorL = e;
		}

		if(scaleL - scaleS < CONV_BISECTION*m_dia)
			break;
	}
}
double POT_SphereGenerator::localFitQ(POT_Sphere* ba)//Ball* ba)
{
	int i;

	float (*point)[3] = m_ps->m_point;
	float (*normal)[3] = m_ps->m_normal;
	float *weight = m_ps->m_weight;

	float c[3] = {ba->m_cx, ba->m_cy, ba->m_cz};
	float r = m_Tqem*ba->m_r;
	int *list, listN;
	m_tree->collectPointIndexInSphere(list, listN, c, r);

	double Q[10] = {0,0,0,0,0,0,0,0,0,0};
	double W = 0;
	double W2 = 0;
	for(i=0; i<listN; i++)
	{
		int j = list[i];
		float* p = point[j];
		float* n = normal[j];

		float vx = p[0] - c[0];
		float vy = p[1] - c[1];
		float vz = p[2] - c[2];
		double w = POT_Sphere::weight(sqrt(vx*vx+vy*vy+vz*vz), r);
		W2 += w;
		w *= weight[j];

		double wnx = w*n[0];
		double wny = w*n[1];
		double wnz = w*n[2];

		Q[0] += wnx*n[0];
		Q[1] += wnx*n[1];
		Q[2] += wnx*n[2];
		Q[3] += wny*n[1];
		Q[4] += wny*n[2];
		Q[5] += wnz*n[2];

		double dot = n[0]*p[0] + n[1]*p[1] + n[2]*p[2];
		Q[6] += dot*wnx;
		Q[7] += dot*wny;
		Q[8] += dot*wnz;

		Q[9] += w*dot*dot;

		W += w;
	}

	if((float)W == 0)
	{
		ba->m_px = c[0];
		ba->m_py = c[1];
		ba->m_pz = c[2];
		return 0;
	}

	double **A = new double*[4];
	double **v = new double*[4];
	double b[4], w[4], x[4];
	for(i=1; i<4; i++)
	{
		A[i] = new double[4];
		v[i] = new double[4];
	}

	W = 1.0/W;
	A[1][1] = Q[0]*W;
	A[1][2] = A[2][1] = Q[1]*W;
	A[1][3] = A[3][1] = Q[2]*W;
	A[2][2] = Q[3]*W;
	A[2][3] = A[3][2] = Q[4]*W;
	A[3][3] = Q[5]*W;

	b[1] = (Q[6] - Q[0]*c[0] - Q[1]*c[1] - Q[2]*c[2])*W;
	b[2] = (Q[7] - Q[1]*c[0] - Q[3]*c[1] - Q[4]*c[2])*W;
	b[3] = (Q[8] - Q[2]*c[0] - Q[4]*c[1] - Q[5]*c[2])*W;

	if(!NRSVD::svdcmp(A, 3, 3, w, v))
	{
		ba->m_px = c[0];
		ba->m_py = c[1];
		ba->m_pz = c[2];
		return 0;
	}

	int k;
	float wmax = (float)fabs(w[1]);
	for(k=2; k<4; k++)
	if(wmax < fabs(w[k]))
	wmax = (float)fabs(w[k]);

	for(k=1;k<4;k++)
	{
		if(fabs(w[k]) < 0.1*wmax) 
			w[k]=0.0;
	}
	NRSVD::svbksb(A, w, v, 3, 3, b, x);

	for(i=1; i<4; i++)
	{
		delete[] A[i];
		delete[] v[i];
	}
	delete[] A;
	delete[] v;

	ba->m_px = c[0] + (float)x[1];
	ba->m_py = c[1] + (float)x[2];
	ba->m_pz = c[2] + (float)x[3];


	double mx = ba->m_px;
	double my = ba->m_py;
	double mz = ba->m_pz;

	double Ax = Q[0]*mx + Q[1]*my + Q[2]*mz;
	double Ay = Q[1]*mx + Q[3]*my + Q[4]*mz;
	double Az = Q[2]*mx + Q[4]*my + Q[5]*mz;

	double err = (Ax-2.0*Q[6])*mx + (Ay-2.0*Q[7])*my + (Az-2.0*Q[8])*mz + Q[9];

	double len = sqrt(x[1]*x[1] + x[2]*x[2] + x[3]*x[3]);
	if(len > ba->m_r)
	{
		ba->m_px = c[0];
		ba->m_py = c[1];
		ba->m_pz = c[2];
	}

	if(listN > m_TTNmin)
		return err; 
	else
		return 0;
}


void POT_SphereGenerator::computeHull(bool* onHull, int* list, int N, float c[3])
{
	//compute base plane
	float n[3];
	computeNormalWithCV(n, list, N);
	float t1[3], t2[3];
	if(fabs(n[0]) < fabs(n[1]))
	{
		double l = sqrt(n[1]*n[1] + n[2]*n[2]);
		t1[0] = 0;
		t1[1] = -(float)(n[2]/l);
		t1[2] = (float)(n[1]/l);
	}
	else
	{
		double l = sqrt(n[0]*n[0] + n[2]*n[2]);
		t1[0] = (float)(n[2]/l);
		t1[1] = 0;
		t1[2] = -(float)(n[0]/l);
	}
	t2[0] = n[1]*t1[2] - n[2]*t1[1];
	t2[1] = n[2]*t1[0] - n[0]*t1[2];
	t2[2] = n[0]*t1[1] - n[1]*t1[0];

	float (*p)[2] = new float[N][2];
	int *index = new int[N];
	int i;
	for(i=0; i<N; i++)
	{
		float* p1 = m_ps->m_point[list[i]];
		p[i][0] = t1[0]*(p1[0]-c[0]) + t1[1]*(p1[1]-c[1]) + t1[2]*(p1[2]-c[2]);
		p[i][1] = t2[0]*(p1[0]-c[0]) + t2[1]*(p1[1]-c[1]) + t2[2]*(p1[2]-c[2]);
		index[i] = i;
		onHull[i] = false;
	}

	int left = 0;
	int right = 0;
	for(i=1; i<N; i++)
	{
		if(p[i][0] < p[left][0])
			left = i;
		else if(p[i][0] > p[right][0])
			right = i;
	}
	onHull[left] = true;
	onHull[right] = true;

	index[0] = left;
	index[left] = 0;
	int tmp = index[N-1];
	index[N-1] = index[right];
	index[right] = tmp;

	quickHull(0, N-1, p, onHull, index);

	for(i=0; i<N; i++)
	index[i] = i;

	index[0] = right;
	index[right] = 0;
	tmp = index[N-1];
	index[N-1] = index[left];
	index[left] = tmp;

	quickHull(0, N-1, p, onHull, index);

	delete[] p;
	delete[] index;
}

void POT_SphereGenerator::quickHull(int s, int e, float (*p)[2], bool* onHull, int* index)
{
	if(s+1 >= e)
		return;
	float *p1 = p[index[s]];
	float *p2 = p[index[e]];
	float nx = p2[1] - p1[1];
	float ny = -(p2[0] - p1[0]);
	int i;
	float max = 0;
	int maxI = s;
	for(i=s+1; i<e; i++)
	{
		float* q = p[index[i]];
		float d = nx*(q[0]-p1[0]) + ny*(q[1]-p1[1]);
		if(d > max)
		{
			max = d;
			maxI = index[i];
		}
	}
	onHull[maxI] = true;

	float *p3 = p[maxI];
	nx = p3[1] - p1[1];
	ny = -(p3[0] - p1[0]);
	i = s+1;
	int j = e-1;
	while(i <= j)
	{
		int k = index[i];
		float* q = p[k];
		if(k != maxI && nx*(q[0]-p1[0]) + ny*(q[1]-p1[1]) > 0)
			i++;
		else
		{
			int tmp = index[i];
			index[i] = index[j];
			index[j] = tmp;
			j--;
		}
	}
	int c1 = i;

	nx = p2[1] - p3[1];
	ny = -(p2[0] - p3[0]);
	i = e-1;
	j = c1;
	while(i >= j)
	{
		int k = index[i];
		float* q = p[k];
		if(k != maxI && nx*(q[0]-p2[0]) + ny*(q[1]-p2[1]) > 0)
		i--;
		else
		{
			int tmp = index[i];
			index[i] = index[j];
			index[j] = tmp;
			j++;
		}
	}
	int c2 = i;

	index[c1] = maxI;
	index[c2] = maxI;

	quickHull(s, c1, p, onHull, index);
	quickHull(c2, e, p, onHull, index);
}


void POT_SphereGenerator::computeNormalWithCV(float n[3], int* list, int N)
{
	int i;
	float (*point)[3] = m_ps->m_point;

	float cx = 0;
	float cy = 0;
	float cz = 0;

	for(i=0; i<N; i++)
	{
		float* p = point[list[i]];
		cx += p[0];
		cy += p[1];
		cz += p[2];
	}
	float Ni = 1.0f/N;
	cx *= Ni;
	cy *= Ni;
	cz *= Ni;

	//data for Jacobi method
	double **A = new double*[4];
	double **v = new double*[4];
	double w[4];
	int nrot;
	for(i=1; i<4; i++)
	{
		A[i] = new double[4];
		A[i][1] = A[i][2] = A[i][3] = 0;
		v[i] = new double[4];
	}

	//CV matrix
	for(i=0; i<N; i++)
	{
		float* p = point[list[i]];

		float vx = p[0] - cx;
		float vy = p[1] - cy;
		float vz = p[2] - cz;

		A[1][1] += vx*vx;
		A[1][2] += vx*vy;
		A[1][3] += vx*vz;

		A[2][2] += vy*vy;
		A[2][3] += vy*vz;

		A[3][3] += vz*vz;
	}
	A[2][1] = A[1][2];
	A[3][1] = A[1][3];
	A[3][2] = A[3][2];

	NRJacobi::Jacobi(A, 3, w, v, &nrot);

	int mini;
	if(fabs(w[1]) < fabs(w[2]))
		mini = 1;
	else
		mini = 2;
	if(fabs(w[mini]) > fabs(w[3]))
		mini = 3;

	n[0] = (float)v[1][mini];
	n[1] = (float)v[2][mini];
	n[2] = (float)v[3][mini];

	for(i=1; i<4; i++)
	{
		delete[] A[i];
		delete[] v[i];
	}
	delete[] A;
	delete[] v;
}

void POT_SphereGenerator::selectSpheres(float eigv)
{

	cout << "." ;

	int i,j, k;
	//Conectivity using spheres
	POT_PointSet *bs = getBalls2();

	clock_t start = clock();

	POT_KdTree* ballTree = new POT_KdTree(bs);

	int baN = bs->m_pointN;

	int** node = new int*[baN];
	int* nodeN = new int[baN];

	for(i=0; i<baN; i++)
		nodeN[i] = 0;

	float (*point)[3] = bs->m_point;
	float *radius = bs->m_weight;

	for(i=0; i<baN; i++)
	{
		float* p = point[i];
		float r = radius[i];

		int listN, *list;
		ballTree->collectPointIndexInSphere(list, listN, p, 2.0f*r);

		for(j=0; j<listN; j++)
		{
			int i1 = list[j];
			if(i == i1)
				continue;
			float* p1 = point[i1];
			float r1 = radius[i1];

			float vx = p[0]-p1[0];
			float vy = p[1]-p1[1];
			float vz = p[2]-p1[2];
			double d = sqrt(vx*vx + vy*vy + vz*vz);

			if(r + r1 > d && fabs(r - r1) < d)
			{
				int* tmp = node[i];
				int nN = nodeN[i];
				int* n = node[i] = new int[nN+1];
				for(k=0; k<nN; k++)
					n[k] = tmp[k];
				if(nN != 0)
					delete[] tmp;
				n[k] = i1;
				nodeN[i]++;

				if(2*r1 < d)
				{
					tmp = node[i1];
					nN = nodeN[i1];
					n = node[i1] = new int[nN+1];
					for(k=0; k<nN; k++)
						n[k] = tmp[k];
					if(nN != 0)
						delete[] tmp;
					n[k] = i;
					nodeN[i1]++;
				}
			}
		}
	}
	delete ballTree;

	float **ivec;
	ivec = new float*[3];
	for(i = 0; i< 3; i++)
	ivec[i] = new float[3];
	float ival[3];
	if(m_isSelected != NULL)
	delete m_isSelected;
	m_isSelected = new bool[baN];
	memset(m_isSelected, 0, baN*sizeof(bool));
	float threshold = eigv;//3.5;
	m_selectNum = 0;
	for(i = 0; i< baN; i++)
	{
		if(nodeN[i]>2)// && nodeN[i]<24)
		{
			PCA(ivec, ival, node[i], bs, nodeN[i]);
			float maxIval;
			maxIval = ival[0];
			if(maxIval<ival[1])	maxIval = ival[1];
			if(maxIval<ival[2]) maxIval = ival[2];

			bool val0, val1, val2;

			val0 = (maxIval/(ival[0]+1.0e-6))>threshold;
			val1 = (maxIval/(ival[1]+1.0e-6))>threshold;
			val2 = (maxIval/(ival[2]+1.0e-6))>threshold;

			if(val0 && val1)
			{
				m_isSelected[i] = true;
				m_selectNum++;
			}
			else if(val0 && val2)
			{
				m_isSelected[i] = true;
				m_selectNum++;
			}
			else if(val1 && val2)
			{
				m_isSelected[i] = true;
				m_selectNum++;
			}
		}
		else// if(nodeN[i]<=2)
		{
			m_isSelected[i] = true;
			m_selectNum++;
		}
	}

	for(i = 0; i< 3; i++)
		delete[] ivec[i];
	delete[] ivec;

	if(m_isSelectedNeigh != NULL)
		delete m_isSelectedNeigh;
	m_isSelectedNeigh = new bool[m_baN];
	memset(m_isSelectedNeigh, 0, m_baN*sizeof(bool));
	m_neighNum = 0;
	for(i = 0; i< m_baN; i++)
	{
		if(m_isSelected[i])
		{
			int j;
			for(j = 0; j< nodeN[i]; j++)
			{
				int ballId = node[i][j];
				if(m_isSelected[ballId])	continue;
				if(m_isSelectedNeigh[ballId])	continue;

				m_isSelectedNeigh[ballId] = true;
					m_neighNum++;
			}
		}
	}

	for(i=0; i<baN; i++)
	if(nodeN[i] != 0)
		delete node[i];
	delete[] node;
	delete[] nodeN;

	m_ballPts = bs;

	m_duration+=clock()-start;
}

void POT_SphereGenerator::splitSpheres()
{
	m_vertPts = getVertices();
	m_ball_out = fopen(BALL_FILE, "w+b");
	m_vertex_out = fopen(VERTEX_FILE, "w+b");

	clock_t start = clock();

	int ballNum = m_ballPts->m_pointN;
	m_baN = ballNum+3*m_selectNum+3*m_neighNum;

	int spaceNum = 4*m_baN;
	float *buff = new float[spaceNum];

	int spaceNum2 = 3*m_baN;
	float *buff2 = new float[spaceNum2];
	int i, k = 0;
	int kk = 0; 
	for(i = 0; i< ballNum; i++)
	{
		float *ct = m_ballPts->m_point[i];
		float *pt = m_vertPts->m_point[i];
		float wt = m_ballPts->m_weight[i];
		if(m_isSelected[i] || m_isSelectedNeigh[i])
		{
			float *nor = m_ballPts->m_normal[i];
			float t1[3], t2[3];
			if(fabs(nor[0]) < fabs(nor[1]))
			{
				double l = sqrt(nor[1]*nor[1] + nor[2]*nor[2]);
				t1[0] = 0;
				t1[1] = -(float)(nor[2]/l);
				t1[2] = (float)(nor[1]/l);
			}
			else{
				double l = sqrt(nor[0]*nor[0] + nor[2]*nor[2]);
				t1[0] = (float)(nor[2]/l);
				t1[1] = 0;
				t1[2] = -(float)(nor[0]/l);
			}
			t2[0] = nor[1]*t1[2] - nor[2]*t1[1];
			t2[1] = nor[2]*t1[0] - nor[0]*t1[2];
			t2[2] = nor[0]*t1[1] - nor[1]*t1[0];

			double len1 = 1.0/sqrt(t1[0]*t1[0]+t1[1]*t1[1]+t1[2]*t1[2]);
			double len2 = 1.0/sqrt(t2[0]*t2[0]+t2[1]*t2[1]+t2[2]*t2[2]);
			t1[0]*=len1;	t1[1]*=len1;	t1[2]*=len1;
			t2[0]*=len2;	t2[1]*=len2;	t2[2]*=len2;


			float rr = 0.5*wt;
			float rad = sqrt(3.0)*rr;
			buff[k++] = ct[0]+rr*t1[0];
			buff[k++] = ct[1]+rr*t1[1];
			buff[k++] = ct[2]+rr*t1[2];
			buff[k++] = rad;
			buff[k++] = ct[0]-rr*t1[0];
			buff[k++] = ct[1]-rr*t1[1];
			buff[k++] = ct[2]-rr*t1[2];
			buff[k++] = rad;
			buff[k++] = ct[0]+rr*t2[0];
			buff[k++] = ct[1]+rr*t2[1];
			buff[k++] = ct[2]+rr*t2[2];
			buff[k++] = rad;
			buff[k++] = ct[0]-rr*t2[0];
			buff[k++] = ct[1]-rr*t2[1];
			buff[k++] = ct[2]-rr*t2[2];
			buff[k++] = rad;


			buff2[kk++] = ct[0]+rr*t1[0];
			buff2[kk++] = ct[1]+rr*t1[1];
			buff2[kk++] = ct[2]+rr*t1[2];
			buff2[kk++] = ct[0]-rr*t1[0];
			buff2[kk++] = ct[1]-rr*t1[1];
			buff2[kk++] = ct[2]-rr*t1[2];
			buff2[kk++] = ct[0]+rr*t2[0];
			buff2[kk++] = ct[1]+rr*t2[1];
			buff2[kk++] = ct[2]+rr*t2[2];
			buff2[kk++] = ct[0]-rr*t2[0];
			buff2[kk++] = ct[1]-rr*t2[1];
			buff2[kk++] = ct[2]-rr*t2[2];
		}
		else
		{
			buff[k++] = ct[0];
			buff[k++] = ct[1];
			buff[k++] = ct[2];
			buff[k++] = wt;

			buff2[kk++] = pt[0];
			buff2[kk++] = pt[1];
			buff2[kk++] = pt[2];
		}

	}

	m_duration+= clock() - start;


	fwrite(buff, sizeof(float), spaceNum, m_ball_out);
	fwrite(buff2, sizeof(float), spaceNum2, m_vertex_out);
	delete[] buff2;
	delete[] buff;

	delete m_ballPts;	m_ballPts = NULL;
	delete m_vertPts;	m_vertPts = NULL;
	delete m_isSelected;	m_isSelected = NULL;
	delete m_isSelectedNeigh; m_isSelectedNeigh = NULL;
	fclose(m_ball_out);
	fclose(m_vertex_out);

}

void POT_SphereGenerator::PCA(float **ivec, float *ival, int *list, POT_PointSet *pts, int N)
{
	int i;
	float (*point)[3] = pts->m_point;

	float cx = 0;
	float cy = 0;
	float cz = 0;

	for(i=0; i<N; i++)
	{
		float* p = point[list[i]];
		cx += p[0];
		cy += p[1];
		cz += p[2];
	}
	float Ni = 1.0f/N;
	cx *= Ni;
	cy *= Ni;
	cz *= Ni;

	//data for Jacobi method
	double **A = new double*[4];
	double **v = new double*[4];
	double w[4];
	int nrot;
	for(i=1; i<4; i++)
	{
		A[i] = new double[4];
		A[i][1] = A[i][2] = A[i][3] = 0;
		v[i] = new double[4];
	}

	//CV matrix
	for(i=0; i<N; i++)
	{
		float* p = point[list[i]];

		float vx = p[0] - cx;
		float vy = p[1] - cy;
		float vz = p[2] - cz;

		A[1][1] += vx*vx;
		A[1][2] += vx*vy;
		A[1][3] += vx*vz;

		A[2][2] += vy*vy;
		A[2][3] += vy*vz;

		A[3][3] += vz*vz;
	}
	A[2][1] = A[1][2];
	A[3][1] = A[1][3];
	A[3][2] = A[3][2];

	NRJacobi::Jacobi(A, 3, w, v, &nrot);

	for(i = 0; i< 3; i++)
	{
		ivec[i][0] = v[1][i+1];
		ivec[i][1] = v[2][i+1];
		ivec[i][2] = v[3][i+1];
		ival[i] = w[i+1];
	}

	for(i=1; i<4; i++)
	{
		delete[] A[i];
		delete[] v[i];
	}
	delete[] A;
	delete[] v;
}
void POT_SphereGenerator::computeNormalOrientationAware()
{
	if(m_ps->m_normal == NULL)
	{
		return;
	}

	int i, j, k;

	int N = m_ps->m_pointN;
	float (*point)[3] = m_ps->m_point;

	float size = m_tree->getLeafSize();
	double *d_array = new double[K];
	int *p_array = new int[K+1];
	for(i=0; i<N; i++)
	{
		float* p = point[i];
		float r = size;
		int listN, *list;
		do{
			m_tree->collectPointIndexInSphere(list, listN, p, r);
			r *= 1.5f;
		}while(listN < K+1);

		for(j=0; j<K; j++)
			d_array[j] = m_dia2;

		for(j=0; j<listN; j++)
		{
			float* q = point[list[j]];
			if(p == q)
			continue;
			float vx = q[0] - p[0];
			float vy = q[1] - p[1];
			float vz = q[2] - p[2];
			double d = vx*vx + vy*vy + vz*vz;

			int index = -1;
			for(k=0; k<K; k++)
			{
				if(d < d_array[k])
				{
					index = k;
					break;
				}
			}
			if(index < 0)
				continue;

			for(k=K-1; k>index; k--)
			{
				p_array[k] = p_array[k-1];
				d_array[k] = d_array[k-1];
			}
			p_array[index] = list[j];
			d_array[index] = d;
		}

		p_array[K] = i;
		float n[3];
		computeNormalWithCVOrientationAware(n, p_array, K+1);
		m_ps->setNormal(i, n[0], n[1], n[2]);
	}
	delete[] p_array;
	delete[] d_array;	
}
void POT_SphereGenerator::computeNormalWithCVOrientationAware(float n[3], int* list, int N)
{
	int i;
	float (*point)[3] = m_ps->m_point;
	float (*normal)[3] = m_ps->m_normal;

	float cx = 0;
	float cy = 0;
	float cz = 0;

	bool *sameOrientation = new bool[N];
	memset(sameOrientation, 0, N*sizeof(bool));
	int count = 0;
	float *currentNormal = normal[list[N-1]];
	for(i=0; i<N; i++)
	{
		float *nn = normal[list[i]];
		if((nn[0]*normal[list[N-1]][0]+nn[1]*normal[list[N-1]][1]+nn[2]*normal[list[N-1]][2])<0)
		{
			continue;
		}
		float* p = point[list[i]];
		cx += p[0];
		cy += p[1];
		cz += p[2];

		count++;
		sameOrientation[i] = true;
	}
	float Ni = 1.0f/count;
	cx *= Ni;
	cy *= Ni;
	cz *= Ni;

	//data for Jacobi method
	double **A = new double*[4];
	double **v = new double*[4];
	double w[4];
	int nrot;
	for(i=1; i<4; i++)
	{
		A[i] = new double[4];
		A[i][1] = A[i][2] = A[i][3] = 0;
		v[i] = new double[4];
	}

	//CV matrix
	for(i=0; i<N; i++)
	{
		if(!sameOrientation[i])
			continue;
		float* p = point[list[i]];

		float vx = p[0] - cx;
		float vy = p[1] - cy;
		float vz = p[2] - cz;

		A[1][1] += vx*vx;
		A[1][2] += vx*vy;
		A[1][3] += vx*vz;

		A[2][2] += vy*vy;
		A[2][3] += vy*vz;

		A[3][3] += vz*vz;
	}
	A[2][1] = A[1][2];
	A[3][1] = A[1][3];
	A[3][2] = A[3][2];

	NRJacobi::Jacobi(A, 3, w, v, &nrot);

	int mini;
	if(fabs(w[1]) < fabs(w[2]))
		mini = 1;
	else
		mini = 2;
	if(fabs(w[mini]) > fabs(w[3]))
		mini = 3;

	n[0] = (float)v[1][mini];
	n[1] = (float)v[2][mini];
	n[2] = (float)v[3][mini];

	if(currentNormal[0]*n[0]+currentNormal[1]*n[1]+currentNormal[2]*n[2]<0)
	{
		n[0] *= -1;	n[1] *= -1;	n[2]*=-1;
	}
	for(i=1; i<4; i++)
	{
		delete[] A[i];
		delete[] v[i];
	}
	delete[] A;
	delete[] v;

	delete[] sameOrientation;
}


int POT_SphereGenerator::removeTempFile()
{
	int rev = 0;
	int result = 0;
	result = remove(BALL_FILE);
	if(result == -1)
		rev = 1;
	result = remove(VERTEX_FILE);
	if(result == -1)
		rev += 2;
	result = remove(NORMAL_FILE);
	if(result == -1)
		rev+=4;

	return rev;
}	